@@ -15,7 +15,11 @@ from typing import (
 )
 
 import sentry_sdk
-from sentry_sdk.ai.utils import set_data_normalized
+from sentry_sdk.ai.utils import (
+    set_data_normalized,
+    truncate_and_annotate_messages,
+    normalize_message_roles,
+)
 from sentry_sdk.consts import OP, SPANDATA
 from sentry_sdk.scope import should_send_default_pii
 from sentry_sdk.utils import (
@@ -462,12 +466,18 @@ def set_span_data_for_request(span, integration, model, contents, kwargs):
             messages.append({"role": "user", "content": contents_text})
 
         if messages:
-            set_data_normalized(
-                span,
-                SPANDATA.GEN_AI_REQUEST_MESSAGES,
-                messages,
-                unpack=False,
+            normalized_messages = normalize_message_roles(messages)
+            scope = sentry_sdk.get_current_scope()
+            messages_data = truncate_and_annotate_messages(
+                normalized_messages, span, scope
             )
+            if messages_data is not None:
+                set_data_normalized(
+                    span,
+                    SPANDATA.GEN_AI_REQUEST_MESSAGES,
+                    messages_data,
+                    unpack=False,
+                )
 
     # Extract parameters directly from config (not nested under generation_config)
     for param, span_key in [
